#include "mainwindow.h"

#include "themewidgetcreator.h"
#include "exceptions.h"

#include "managers/greetermanagers.h"
#include "widgets/basethemewidget.h"
#include "managers/sessionsmanager.h"
#include "managers/usersmanager.h"
#include "managers/shutdownmanager.h"
#include "managers/virtualkeyboardmanager.h"
#include "managers/keyboardmanager.h"
#include "managers/connectionmanager.h"
#include "managers/consoleauthmanager.h"
#include "managers/localauthmanager.h"
#include "managers/domainmanager.h"

#include <QMessageBox>
#include <QGridLayout>
#include <QLocale>
#include <QDir>
#include <QStandardPaths>
#include <QDebug>


MainWindow::MainWindow(QWidget* parent) :
    QDialog(parent)
{
    setObjectName("mainDialog");
}

MainWindow::~MainWindow()
{
    delete m_managers;
}

void MainWindow::load()
{
    try {
        ThemeInfo themeInfo(QDir::current().filePath(QFileInfo(QString("../") + __FILE__).dir().filePath("../themes.17/themes/fly-modern")));
        themeInfo.read();

        const QString pluginPath(QDir::current().filePath("../../dm_greet_modern/debug/libfly-dmgreet_modern.so"));

        UserInfo::Filter usersFilter;
        usersFilter.showUsers = true;
        usersFilter.lowUserId = 1000;
        usersFilter.highUserId = 2000;
        usersFilter.showRoot = false;
        usersFilter.sortUsers = true;
        auto users = UserInfo::getUserList(usersFilter);
        const auto icons = UserIcon::getUserIcons("/usr/share/fly-dm/faces", FaceSource::UserSystem);
        const auto usersManager = new UsersManager(users, icons, false, true, "", UsersManager::PreselUser::Prev, true, UsersManager::FirstScreen::Auto);

        QSettings dmrc("/etc/X11/fly-dm/fly-dmrc", QSettings::IniFormat);
        dmrc.beginGroup("X-*-Core");
        const auto sessionsDescriptions = SessionReader::readDirs(dmrc.value("SessionsDirs").toStringList());
        SessionList sessions;
        std::copy_if(sessionsDescriptions.begin(), sessionsDescriptions.end(), std::back_inserter(sessions), [](const SessionDescription& session) {
            if (session.hidden) {
                return false;
            }

            if (!session.tryExec.isEmpty()) {
                return !QStandardPaths::findExecutable(session.tryExec).isEmpty();
            }

            return true;
        });
        const auto sessionsManager = new SessionsManager(sessions);

        ConnectionManager::ConnectionData connectionData;
        connectionData.allowClose = true;
        const auto connectionManager = new ConnectionManager(connectionData);

        m_managers = new GreeterManagers({new LocalAuthManager(),
                                          sessionsManager,
                                          usersManager,
                                          new ShutdownManager(false),
                                          new VirtualKeyboardManager(""),
                                          new KeyboardManager(XKeyboard::instance()),
                                          connectionManager,
                                          new ConsoleAuthManager(true)});

        const auto locale = QLocale().name();
        SystemInfo info;
        info.locale = locale.size() >= 2 ? locale.left(2) : QString();
        info.loginPlugins.append(pluginPath);
        info.dname = ":1";
        info.stsFile = "/var/lib/fly-dm/fly-dmsts";

        const auto mainThemeWidget = ThemeWidgetCreator::create(themeInfo, info, m_managers);

        const auto layout = new QGridLayout();
        layout->setMargin(0);
        layout->addWidget(mainThemeWidget);
        setLayout(layout);
    } catch (const DetailsException& exception) {
        QMessageBox::critical(this,
                              tr("Error"),
                              exception.fullDetails());
    }
}
